﻿"use strict";

window.Wingtip = window.Wingtip || {};
Wingtip.CrossDomain = function () {

    var hostweburl,
        appweburl,

    init = function () {
        hostweburl = getQueryStringParameter("SPHostUrl");
        appweburl = getQueryStringParameter("SPAppWebUrl");
    },

    load = function () {
        
        var executor = new SP.RequestExecutor(appweburl);

        executor.executeAsync(
        {
            url:
            appweburl +
            "/_api/SP.AppContextSite(@target)/web/lists/getByTitle('Contacts')/items" +
                            "?@target='" + hostweburl + "'" +
                            "&$select=Id,FirstName,Title,WorkPhone,Email" +
                            "&$orderby=Title,FirstName",
            method: "GET",
            headers: { "accept": "application/json;odata=verbose" },
            success: successHandler,
            error: errorHandler
        })
    },

    getFormDigest = function () {

        var deferred = $.Deferred();
        var executor = new SP.RequestExecutor(appweburl);

        executor.executeAsync(
        {
            url:
            appweburl + "/_api/contextinfo",
            method: "POST",
            headers: {
                "accept": "application/json;odata=verbose",
            },
            success: function (data) {
                deferred.resolve(JSON.parse(data.body).d.GetContextWebInformation.FormDigestValue);
            },
            error: function (data, errorCode, errorMessage) {
                deferred.reject(data, errorCode, errorMessage);
            }
        });

        return deferred.promise();
    },

    update = function (id, firstName, lastName) {
        
        getFormDigest().then(
            function (formDigest) {
                var executor = new SP.RequestExecutor(appweburl);

                executor.executeAsync(
                {
                    url:
                    appweburl +
                    "/_api/SP.AppContextSite(@target)/web/lists/getByTitle('Contacts')/getItemByStringId('" + id + "')" +
                    "?@target='" + hostweburl + "'",
                    method: "POST",
                    headers: {
                        "accept": "application/json;odata=verbose",
                        "content-type": "application/json;odata=verbose",
                        "X-RequestDigest": formDigest,
                        "IF-MATCH": "*",
                        "X-Http-Method": "PATCH"
                    },
                    body: JSON.stringify(
                    {
                        '__metadata': {
                            'type': "SP.Data.ContactsListItem"
                        },
                        'Title': lastName,
                        'FirstName': firstName
                    }),
                    success: load,
                    error: errorHandler
                });

            },
            function (data, errorCode, errorMessage) {
                errorHandler(data, errorCode, errorMessage);
            }
        );

    },

    successHandler = function (data) {

        var jsonObject = JSON.parse(data.body);
        var results = jsonObject.d.results;

        var contacts = Wingtip.ContactViewModel.get_contacts();
        contacts.removeAll();

        for (var i = 0; i < results.length; i++) {
            contacts.push(
                new Wingtip.Contact(
                results[i].Id,
                results[i].Title,
                results[i].FirstName,
                results[i].WorkPhone,
                results[i].Email));
        }


    },

    errorHandler = function (data, errorCode, errorMessage) {
        alert("Error calling cross-domain: " + errorMessage);
    },

    getQueryStringParameter = function (name) {
        try {
            var args = window.location.search.substring(1).split("&");
            var r = "";
            for (var i = 0; i < args.length; i++) {
                var n = args[i].split("=");
                if (n[0] == name)
                    r = decodeURIComponent(n[1]);
            }
            return r;
        }
        catch (err) {
            return 'undefined';
        }
    };

    return {
        init: init,
        load: load,
        update: update
    }

}()

